@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2015, Vispy Development Team.
+# Copyright (c) Vispy Development Team. All Rights Reserved.
 # Distributed under the (new) BSD License. See LICENSE.txt for more info.
 
 """
@@ -31,7 +31,8 @@ def unixTime():
     """
     return systime.time()
 
-if sys.platform.startswith('win'):
+
+if sys.platform.startswith('win') and sys.version < '3.3':
     cstart = systime.clock()  # Required to start the clock in windows
     START_TIME = systime.time() - cstart
 
