@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # -----------------------------------------------------------------------------
-# Copyright (c) 2015, Vispy Development Team.
+# Copyright (c) Vispy Development Team. All Rights Reserved.
 # Distributed under the (new) BSD License. See LICENSE.txt for more info.
 # -----------------------------------------------------------------------------
 """
@@ -11,18 +11,18 @@ import numpy as np
 
 from ..color import ColorArray
 from ..gloo import VertexBuffer, _check_valid
-from .shaders import ModularProgram, Function, Variable
+from .shaders import Function, Variable
 from .visual import Visual
 
 
 vert = """
-uniform mat4 u_projection;
 uniform float u_antialias;
-uniform int u_px_scale;
+uniform float u_px_scale;
+uniform float u_scale;
 
-attribute vec3  a_position;
-attribute vec4  a_fg_color;
-attribute vec4  a_bg_color;
+attribute vec3 a_position;
+attribute vec4 a_fg_color;
+attribute vec4 a_bg_color;
 attribute float a_edgewidth;
 attribute float a_size;
 
@@ -32,19 +32,19 @@ varying float v_edgewidth;
 varying float v_antialias;
 
 void main (void) {
-    $v_size = a_size * u_px_scale;
-    v_edgewidth = a_edgewidth;
+    $v_size = a_size * u_px_scale * u_scale;
+    v_edgewidth = a_edgewidth * float(u_px_scale);
     v_antialias = u_antialias;
     v_fg_color  = a_fg_color;
     v_bg_color  = a_bg_color;
     gl_Position = $transform(vec4(a_position,1.0));
     float edgewidth = max(v_edgewidth, 1.0);
-    gl_PointSize = $scalarsize($v_size) + 4*(edgewidth + 1.5*v_antialias);
+    gl_PointSize = ($v_size) + 4.*(edgewidth + 1.5*v_antialias);
 }
 """
 
 
-frag = """
+frag = """#version 120
 varying vec4 v_fg_color;
 varying vec4 v_bg_color;
 varying float v_edgewidth;
@@ -52,10 +52,14 @@ varying float v_antialias;
 
 void main()
 {
+    // Discard plotting marker body and edge if zero-size
+    if ($v_size <= 0.)
+        discard;
+
     float edgewidth = max(v_edgewidth, 1.0);
     float edgealphafactor = min(v_edgewidth, 1.0);
 
-    float size = $scalarsize($v_size) + 4*(edgewidth + 1.5*v_antialias);
+    float size = $v_size + 4.*(edgewidth + 1.5*v_antialias);
     // factor 6 for acute edge angles that need room as for star marker
 
     // The marker function needs to be linked with this shader
@@ -103,7 +107,7 @@ void main()
         {
             float alpha = d/v_antialias;
             alpha = exp(-alpha*alpha);
-            if (r > 0)
+            if (r > 0.)
             {
                 // outer part of the edge: fade out into the background...
                 gl_FragColor = vec4(edgecolor.rgb, alpha*edgecolor.a);
@@ -117,25 +121,11 @@ void main()
 }
 """
 
-size1d = """
-float size1d(float size)
-{
-    return size;
-}
-"""
-
-size2d = """
-float size2d(vec2 size)
-{
-    return max(size.x, size.y);
-}
-"""
-
 disc = """
 float disc(vec2 pointcoord, float size)
 {
     float r = length((pointcoord.xy - vec2(0.5,0.5))*size);
-    r -= $v_size/2;
+    r -= $v_size/2.;
     return r;
 }
 """
@@ -159,8 +149,8 @@ float rect(vec2 pointcoord, float size)
 ring = """
 float ring(vec2 pointcoord, float size)
 {
-    float r1 = length((pointcoord.xy - vec2(0.5,0.5))*size) - $v_size/2;
-    float r2 = length((pointcoord.xy - vec2(0.5,0.5))*size) - $v_size/4;
+    float r1 = length((pointcoord.xy - vec2(0.5,0.5))*size) - $v_size/2.;
+    float r2 = length((pointcoord.xy - vec2(0.5,0.5))*size) - $v_size/4.;
     float r = max(r1,-r2);
     return r;
 }
@@ -195,7 +185,7 @@ square = """
 float square(vec2 pointcoord, float size)
 {
     float r = max(abs(pointcoord.x -.5)*size, abs(pointcoord.y -.5)*size);
-    r -= $v_size/2;
+    r -= $v_size/2.;
     return r;
 }
 """
@@ -206,12 +196,12 @@ float x_(vec2 pointcoord, float size)
     vec2 rotcoord = vec2((pointcoord.x + pointcoord.y - 1.) / sqrt(2.),
                          (pointcoord.y - pointcoord.x) / sqrt(2.));
     //vbar
-    float r1 = abs(rotcoord.x)*size - $v_size/6;
-    float r2 = abs(rotcoord.y)*size - $v_size/2;
+    float r1 = abs(rotcoord.x)*size - $v_size/6.;
+    float r2 = abs(rotcoord.y)*size - $v_size/2.;
     float vbar = max(r1,r2);
     //hbar
-    float r3 = abs(rotcoord.y)*size - $v_size/6;
-    float r4 = abs(rotcoord.x)*size - $v_size/2;
+    float r3 = abs(rotcoord.y)*size - $v_size/6.;
+    float r4 = abs(rotcoord.x)*size - $v_size/2.;
     float hbar = max(r3,r4);
     return min(vbar, hbar);
 }
@@ -222,7 +212,7 @@ diamond = """
 float diamond(vec2 pointcoord, float size)
 {
     float r = abs(pointcoord.x -.5)*size + abs(pointcoord.y -.5)*size;
-    r -= $v_size/2;
+    r -= $v_size/2.;
     return r / sqrt(2.);//account for slanted edge and correct for width
 }
 """
@@ -231,8 +221,8 @@ float diamond(vec2 pointcoord, float size)
 vbar = """
 float vbar(vec2 pointcoord, float size)
 {
-    float r1 = abs(pointcoord.x - 0.5)*size - $v_size/6;
-    float r3 = abs(pointcoord.y - 0.5)*size - $v_size/2;
+    float r1 = abs(pointcoord.x - 0.5)*size - $v_size/6.;
+    float r3 = abs(pointcoord.y - 0.5)*size - $v_size/2.;
     float r = max(r1,r3);
     return r;
 }
@@ -241,8 +231,8 @@ float vbar(vec2 pointcoord, float size)
 hbar = """
 float rect(vec2 pointcoord, float size)
 {
-    float r2 = abs(pointcoord.y - 0.5)*size - $v_size/6;
-    float r3 = abs(pointcoord.x - 0.5)*size - $v_size/2;
+    float r2 = abs(pointcoord.y - 0.5)*size - $v_size/6.;
+    float r3 = abs(pointcoord.x - 0.5)*size - $v_size/2.;
     float r = max(r2,r3);
     return r;
 }
@@ -252,12 +242,12 @@ cross = """
 float cross(vec2 pointcoord, float size)
 {
     //vbar
-    float r1 = abs(pointcoord.x - 0.5)*size - $v_size/6;
-    float r2 = abs(pointcoord.y - 0.5)*size - $v_size/2;
+    float r1 = abs(pointcoord.x - 0.5)*size - $v_size/6.;
+    float r2 = abs(pointcoord.y - 0.5)*size - $v_size/2.;
     float vbar = max(r1,r2);
     //hbar
-    float r3 = abs(pointcoord.y - 0.5)*size - $v_size/6;
-    float r4 = abs(pointcoord.x - 0.5)*size - $v_size/2;
+    float r3 = abs(pointcoord.y - 0.5)*size - $v_size/6.;
+    float r4 = abs(pointcoord.x - 0.5)*size - $v_size/2.;
     float hbar = max(r3,r4);
     return min(vbar, hbar);
 }
@@ -337,10 +327,10 @@ float rect(vec2 pointcoord, float size)
     const float PI2_5 = 3.141592653589*2./5.;
     const float PI2_20 = 3.141592653589/10.;  //PI*2/20
     // downwards shift to that the marker center is halfway vertically
-    // between the top of the upward spike (y = -v_size/2)
+    // between the top of the upward spike (y = -v_size/2.)
     // and the bottom of one of two downward spikes
-    // (y = +v_size/2*cos(2*pi/10) approx +v_size/2*0.8)
-    // center is at -v_size/2*0.1
+    // (y = +v_size/2.*cos(2.*pi/10.) approx +v_size/2.*0.8)
+    // center is at -v_size/2.*0.1
     float shift_y = -0.05*$v_size;
     // first spike upwards,
     // rotate spike by 72 deg four times to complete the star
@@ -349,7 +339,7 @@ float rect(vec2 pointcoord, float size)
         //if not the first spike, rotate it upwards
         float x = (pointcoord.x - 0.5)*size;
         float y = (pointcoord.y - 0.5)*size;
-        float spike_rot_angle = i*PI2_5;
+        float spike_rot_angle = float(i) * PI2_5;
         float cosangle = cos(spike_rot_angle);
         float sinangle = sin(spike_rot_angle);
         float spike_x = x;
@@ -364,7 +354,7 @@ float rect(vec2 pointcoord, float size)
         // (point whose coords are -s/2, 0 where s is the size of the marker)
         // compute y coordonates as well because
         // we do a second rotation to put the spike at its final position
-        float rot_center_y = -$v_size/2;
+        float rot_center_y = -$v_size/2.;
         float rot18x = cos(PI2_20) * spike_x
                             - sin(PI2_20) * (spike_y - rot_center_y);
         //rotate -18 deg the zone x > 0 arount the top of the star
@@ -487,7 +477,7 @@ _marker_dict = {
     '>': arrow,
     '^': triangle_up,
     'v': triangle_down,
-    '*': star
+    '*': star,
 }
 marker_types = tuple(sorted(list(_marker_dict.keys())))
 
@@ -495,16 +485,23 @@ marker_types = tuple(sorted(list(_marker_dict.keys())))
 class MarkersVisual(Visual):
     """ Visual displaying marker symbols.
     """
-    def __init__(self):
-        self._program = ModularProgram(vert, frag)
+    def __init__(self, **kwargs):
+        self._vbo = VertexBuffer()
         self._v_size_var = Variable('varying float v_size')
-        self._program.vert['v_size'] = self._v_size_var
-        self._program.frag['v_size'] = self._v_size_var
-        self._program.vert['scalarsize'] = Function(size1d)
-        self._program.frag['scalarsize'] = Function(size1d)
-        Visual.__init__(self)
-        self.set_gl_state(depth_test=False, blend=True,
+        self._symbol = None
+        self._marker_fun = None
+        self._data = None
+        self.antialias = 1
+        self.scaling = False
+        Visual.__init__(self, vcode=vert, fcode=frag)
+        self.shared_program.vert['v_size'] = self._v_size_var
+        self.shared_program.frag['v_size'] = self._v_size_var
+        self.set_gl_state(depth_test=True, blend=True,
                           blend_func=('src_alpha', 'one_minus_src_alpha'))
+        self._draw_mode = 'points'
+        if len(kwargs) > 0:
+            self.set_data(**kwargs)
+        self.freeze()
 
     def set_data(self, pos=None, symbol='o', size=10., edge_width=1.,
                  edge_width_rel=None, edge_color='black', face_color='white',
@@ -537,8 +534,6 @@ class MarkersVisual(Visual):
         vbar, hbar, cross, tailed_arrow, x, triangle_up, triangle_down,
         and star.
         """
-        assert (isinstance(pos, np.ndarray) and
-                pos.ndim == 2 and pos.shape[1] in (2, 3))
         if (edge_width is not None) + (edge_width_rel is not None) != 1:
             raise ValueError('exactly one of edge_width and edge_width_rel '
                              'must be non-None')
@@ -548,7 +543,7 @@ class MarkersVisual(Visual):
         else:
             if edge_width_rel < 0:
                 raise ValueError('edge_width_rel cannot be negative')
-        self.set_symbol(symbol)
+        self.symbol = symbol
         self.scaling = scaling
 
         edge_color = ColorArray(edge_color).rgba
@@ -559,86 +554,82 @@ class MarkersVisual(Visual):
         if len(face_color) == 1:
             face_color = face_color[0]
 
-        n = len(pos)
-        data = np.zeros(n, dtype=[('a_position', np.float32, 3),
-                                  ('a_fg_color', np.float32, 4),
-                                  ('a_bg_color', np.float32, 4),
-                                  ('a_size', np.float32, 1),
-                                  ('a_edgewidth', np.float32, 1)])
-        data['a_fg_color'] = edge_color
-        data['a_bg_color'] = face_color
-        if edge_width is not None:
-            data['a_edgewidth'] = edge_width
-        else:
-            data['a_edgewidth'] = size*edge_width_rel
-        data['a_position'][:, :pos.shape[1]] = pos
-        data['a_size'] = size
-        self.antialias = 1.
-        self._data = data
-        self._vbo = VertexBuffer(data)
+        if pos is not None:
+            assert (isinstance(pos, np.ndarray) and
+                    pos.ndim == 2 and pos.shape[1] in (2, 3))
+
+            n = len(pos)
+            data = np.zeros(n, dtype=[('a_position', np.float32, 3),
+                                      ('a_fg_color', np.float32, 4),
+                                      ('a_bg_color', np.float32, 4),
+                                      ('a_size', np.float32),
+                                      ('a_edgewidth', np.float32)])
+            data['a_fg_color'] = edge_color
+            data['a_bg_color'] = face_color
+            if edge_width is not None:
+                data['a_edgewidth'] = edge_width
+            else:
+                data['a_edgewidth'] = size*edge_width_rel
+            data['a_position'][:, :pos.shape[1]] = pos
+            data['a_size'] = size
+            self.shared_program['u_antialias'] = self.antialias  # XXX make prop
+            self._data = data
+            if self._symbol is not None:
+                # If we have no symbol set, we skip drawing (_prepare_draw
+                # returns False). This causes the GLIR queue to not flush,
+                # and thus the GLIR queue fills with VBO DATA commands, resulting
+                # in a "memory leak". Thus only set the VertexBuffer data if we
+                # are actually going to draw.
+                self._vbo.set_data(data)
+                self.shared_program.bind(self._vbo)
+
         self.update()
 
-    def set_symbol(self, symbol='o'):
-        """Set the symbol
+    @property
+    def symbol(self):
+        return self._symbol
 
-        Parameters
-        ----------
-        symbol : str
-            The symbol.
-        """
-        _check_valid('symbol', symbol, marker_types)
-        self._marker_fun = Function(_marker_dict[symbol])
-        self._marker_fun['v_size'] = self._v_size_var
-        self._program.frag['marker'] = self._marker_fun
+    @symbol.setter
+    def symbol(self, symbol):
+        if symbol == self._symbol:
+            return
+        if (symbol is not None and self._symbol is None and
+                self._data is not None):
+            # Allow user to configure symbol after a set_data call with
+            # symbol=None. This can break down if the user does a consecutive
+            # marker.symbol = 'disc'
+            # marker.symbol = None
+            # without drawing. At this point the memory leaking ensues
+            # but this case is unlikely/makes no sense.
+            self._vbo.set_data(self._data)
+            self.shared_program.bind(self._vbo)
+        self._symbol = symbol
+        if symbol is None:
+            self._marker_fun = None
+        else:
+            _check_valid('symbol', symbol, marker_types)
+            self._marker_fun = Function(_marker_dict[symbol])
+            self._marker_fun['v_size'] = self._v_size_var
+            self.shared_program.frag['marker'] = self._marker_fun
+        self.update()
 
-    def draw(self, transforms):
-        """Draw the visual
+    def _prepare_transforms(self, view):
+        xform = view.transforms.get_transform()
+        view.view_program.vert['transform'] = xform
 
-        Parameters
-        ----------
-        transforms : instance of TransformSystem
-            The transforms to use.
-        """
-        Visual.draw(self, transforms)
-
-        xform = transforms.get_full_transform()
-        self._program.vert['transform'] = xform
-        # TO DO: find a way to avoid copying data and rebinding them to the vbo
+    def _prepare_draw(self, view):
+        if self._symbol is None:
+            return False
+        view.view_program['u_px_scale'] = view.transforms.pixel_scale
         if self.scaling:
-            update_data = self._data.copy()
-            # 0.5 factor due to the difference between the viewbox spanning
-            # [-1, 1] in the framebuffer coordinates intervals
-            # and the viewbox spanning [0, 1] intervals
-            # in the Visual coordinates
-            # TO DO: find a way to get the scale directly
-            scale = (
-                0.5*transforms.visual_to_document.simplified().scale[:2] *
-                transforms.document_to_framebuffer.simplified().scale[:2] *
-                transforms.framebuffer_to_render.simplified().scale[:2]
-            )
-            update_data['a_size'] *= min(scale)
-            update_data['a_edgewidth'] *= min(scale)
-            self._vbo.set_data(update_data)
-        self._program.prepare()
-        self._program['u_antialias'] = self.antialias
+            tr = view.transforms.get_transform('visual', 'document').simplified
+            mat = tr.map(np.eye(3)) - tr.map(np.zeros((3, 3)))
+            scale = np.linalg.norm(mat[:, :3])
+            view.view_program['u_scale'] = scale
+        else:
+            view.view_program['u_scale'] = 1
 
-        d2f = transforms.document_to_framebuffer
-        self._program['u_px_scale'] = (d2f.map((1, 0)) - d2f.map((0, 0)))[0]
-        self._program.bind(self._vbo)
-        self._program.draw('points')
-
-    def bounds(self, mode, axis):
-        """Get the bounds
-
-        Parameters
-        ----------
-        mode : str
-            Describes the type of boundary requested. Can be "visual", "data",
-            or "mouse".
-        axis : 0, 1, 2
-            The axis along which to measure the bounding values, in
-            x-y-z order.
-        """
+    def _compute_bounds(self, axis, view):
         pos = self._data['a_position']
         if pos is None:
             return None
